@@ -3,7 +3,7 @@
 /*  padding.c -- depad subcommand.
 
     Copyright (C) 2011, 2012 Broad Institute.
-    Copyright (C) 2014 Genome Research Ltd.
+    Copyright (C) 2014, 2015 Genome Research Ltd.
     Portions copyright (C) 2012, 2013 Peter Cock, The James Hutton Institute.
 
     Author: Heng Li <lh3@sanger.ac.uk>
@@ -29,26 +29,30 @@ DEALINGS IN THE SOFTWARE.  */
 #include <string.h>
 #include <assert.h>
 #include <unistd.h>
-#include "htslib/kstring.h"
+#include <htslib/kstring.h>
+#include <htslib/sam.h>
+#include <htslib/faidx.h>
 #include "sam_header.h"
-#include "sam.h"
-#include "bam.h"
-#include "htslib/faidx.h"
+#include "sam_opts.h"
 
-bam_header_t *bam_header_dup(const bam_header_t *h0); /*in sam.c*/
+#define bam_reg2bin(b,e) hts_reg2bin((b),(e), 14, 5)
+
+// The one and only function needed from sam.c.
+// Explicitly here to avoid including bam.h translation layer.
+extern char *samfaipath(const char *fn_ref);
 
 static void replace_cigar(bam1_t *b, int n, uint32_t *cigar)
 {
     if (n != b->core.n_cigar) {
         int o = b->core.l_qname + b->core.n_cigar * 4;
-        if (b->data_len + (n - b->core.n_cigar) * 4 > b->m_data) {
-            b->m_data = b->data_len + (n - b->core.n_cigar) * 4;
+        if (b->l_data + (n - b->core.n_cigar) * 4 > b->m_data) {
+            b->m_data = b->l_data + (n - b->core.n_cigar) * 4;
             kroundup32(b->m_data);
             b->data = (uint8_t*)realloc(b->data, b->m_data);
         }
-        memmove(b->data + b->core.l_qname + n * 4, b->data + o, b->data_len - o);
+        memmove(b->data + b->core.l_qname + n * 4, b->data + o, b->l_data - o);
         memcpy(b->data + b->core.l_qname, cigar, n * 4);
-        b->data_len += (n - b->core.n_cigar) * 4;
+        b->l_data += (n - b->core.n_cigar) * 4;
         b->core.n_cigar = n;
     } else memcpy(b->data + b->core.l_qname, cigar, n * 4);
 }
@@ -61,42 +65,45 @@ static void replace_cigar(bam1_t *b, int n, uint32_t *cigar)
         _c[_n++] = (_v); \
     } while (0)
 
-static void unpad_seq(bam1_t *b, kstring_t *s)
+static int unpad_seq(bam1_t *b, kstring_t *s)
 {
+    // Returns 0 on success, -1 on an error
     int k, j, i;
     int length;
-    uint32_t *cigar = bam1_cigar(b);
-    uint8_t *seq = bam1_seq(b);
+    int cigar_n_warning = 0; /* Make this a global and limit to one CIGAR N warning? */
+    uint32_t *cigar = bam_get_cigar(b);
+    uint8_t *seq = bam_get_seq(b);
+
     // b->core.l_qseq gives length of the SEQ entry (including soft clips, S)
     // We need the padded length after alignment from the CIGAR (excluding
     // soft clips S, but including pads from CIGAR D operations)
-    length = 0;
-    for (k = 0; k < b->core.n_cigar; ++k) {
-        int op, ol;
-        op= bam_cigar_op(cigar[k]);
-        ol = bam_cigar_oplen(cigar[k]);
-        if (op == BAM_CMATCH || op == BAM_CEQUAL || op == BAM_CDIFF || op == BAM_CDEL)
-            length += ol;
-    }
+    length = bam_cigar2rlen(b->core.n_cigar, cigar);
     ks_resize(s, length);
     for (k = 0, s->l = 0, j = 0; k < b->core.n_cigar; ++k) {
         int op, ol;
         op = bam_cigar_op(cigar[k]);
         ol = bam_cigar_oplen(cigar[k]);
         if (op == BAM_CMATCH || op == BAM_CEQUAL || op == BAM_CDIFF) {
-            for (i = 0; i < ol; ++i, ++j) s->s[s->l++] = bam1_seqi(seq, j);
+            for (i = 0; i < ol; ++i, ++j) s->s[s->l++] = bam_seqi(seq, j);
         } else if (op == BAM_CSOFT_CLIP) {
             j += ol;
         } else if (op == BAM_CHARD_CLIP) {
             /* do nothing */
         } else if (op == BAM_CDEL) {
             for (i = 0; i < ol; ++i) s->s[s->l++] = 0;
+        } else if (op == BAM_CREF_SKIP) {
+            /* Treat CIGAR N as D (not ideal, but better than ignoring it) */
+            for (i = 0; i < ol; ++i) s->s[s->l++] = 0;
+            if (0 == cigar_n_warning) {
+                cigar_n_warning = -1;
+                fprintf(pysamerr, "[depad] WARNING: CIGAR op N treated as op D in read %s\n", bam_get_qname(b));
+            }
         } else {
-            fprintf(pysamerr, "[depad] ERROR: Didn't expect CIGAR op %c in read %s\n", BAM_CIGAR_STR[op], bam1_qname(b));
-            exit(1);
+            fprintf(pysamerr, "[depad] ERROR: Didn't expect CIGAR op %c in read %s\n", BAM_CIGAR_STR[op], bam_get_qname(b));
+            return -1;
         }
     }
-    assert(length == s->l);
+    return length != s->l;
 }
 
 int load_unpadded_ref(faidx_t *fai, char *ref_name, int ref_len, kstring_t *seq)
@@ -119,7 +126,7 @@ int load_unpadded_ref(faidx_t *fai, char *ref_name, int ref_len, kstring_t *seq)
             // Map gaps to null to match unpad_seq function
             seq->s[seq->l++] = 0;
         } else {
-            int i = bam_nt16_table[(int)base];
+            int i = seq_nt16_table[(int)base];
             if (i == 0 || i==16) { // Equals maps to 0, anything unexpected to 16
                 fprintf(pysamerr, "[depad] ERROR: Invalid character %c (ASCII %i) in FASTA sequence %s\n", base, (int)base, ref_name);
                 free(fai_ref);
@@ -152,7 +159,7 @@ int get_unpadded_len(faidx_t *fai, char *ref_name, int padded_len)
         if (base == '-' || base == '*') {
             gaps += 1;
         } else {
-            int i = bam_nt16_table[(int)base];
+            int i = seq_nt16_table[(int)base];
             if (i == 0 || i==16) { // Equals maps to 0, anything unexpected to 16
                 fprintf(pysamerr, "[depad] ERROR: Invalid character %c (ASCII %i) in FASTA sequence '%s'\n", base, (int)base, ref_name);
                 free(fai_ref);
@@ -177,9 +184,8 @@ static inline int * update_posmap(int *posmap, kstring_t ref)
     return posmap;
 }
 
-int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
+int bam_pad2unpad(samFile *in, samFile *out,  bam_hdr_t *h, faidx_t *fai)
 {
-    bam_header_t *h = 0;
     bam1_t *b = 0;
     kstring_t r, q;
     int r_tid = -1;
@@ -189,16 +195,22 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
     b = bam_init1();
     r.l = r.m = q.l = q.m = 0; r.s = q.s = 0;
     int read_ret;
-    h = in->header;
-    while ((read_ret = samread(in, b)) >= 0) { // read one alignment from `in'
-        uint32_t *cigar = bam1_cigar(b);
+    while ((read_ret = sam_read1(in, h, b)) >= 0) { // read one alignment from `in'
+        // Cannot depad unmapped CRAM data
+        if (b->core.flag & BAM_FUNMAP)
+            goto next_seq;
+
+        uint32_t *cigar = bam_get_cigar(b);
         n2 = 0;
-        if (b->core.pos == 0 && b->core.tid >= 0 && strcmp(bam1_qname(b), h->target_name[b->core.tid]) == 0) {
-            // fprintf(pysamerr, "[depad] Found embedded reference '%s'\n", bam1_qname(b));
+        if (b->core.pos == 0 && b->core.tid >= 0 && strcmp(bam_get_qname(b), h->target_name[b->core.tid]) == 0) {
+            // fprintf(pysamerr, "[depad] Found embedded reference '%s'\n", bam_get_qname(b));
             r_tid = b->core.tid;
-            unpad_seq(b, &r);
+            if (0!=unpad_seq(b, &r)) {
+                fprintf(pysamerr, "[depad] ERROR: Problem parsing SEQ and/or CIGAR in reference %s\n", bam_get_qname(b));
+                return -1;
+            };
             if (h->target_len[r_tid] != r.l) {
-                fprintf(pysamerr, "[depad] ERROR: (Padded) length of '%s' is %u in BAM header, but %llu in embedded reference\n", bam1_qname(b), h->target_len[r_tid], (unsigned long long)(r.l));
+                fprintf(pysamerr, "[depad] ERROR: (Padded) length of '%s' is %u in BAM header, but %llu in embedded reference\n", bam_get_qname(b), h->target_len[r_tid], (unsigned long long)(r.l));
                 return -1;
             }
             if (fai) {
@@ -214,8 +226,8 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
                         // Show gaps as ASCII 45
                         fprintf(pysamerr, "[depad] ERROR: Embedded sequence and reference FASTA don't match for %s base %i, '%c' vs '%c'\n",
                             h->target_name[b->core.tid], i+1,
-                            r.s[i] ? bam_nt16_rev_table[(int)r.s[i]] : 45,
-                            q.s[i] ? bam_nt16_rev_table[(int)q.s[i]] : 45);
+                            r.s[i] ? seq_nt16_str[(int)r.s[i]] : 45,
+                            q.s[i] ? seq_nt16_str[(int)q.s[i]] : 45);
                         return -1;
                     }
                 }
@@ -226,11 +238,11 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
         } else if (b->core.n_cigar > 0) {
             int i, k, op;
             if (b->core.tid < 0) {
-                fprintf(pysamerr, "[depad] ERROR: Read '%s' has CIGAR but no RNAME\n", bam1_qname(b));
+                fprintf(pysamerr, "[depad] ERROR: Read '%s' has CIGAR but no RNAME\n", bam_get_qname(b));
                 return -1;
             } else if (b->core.tid == r_tid) {
                 ; // good case, reference available
-                //fprintf(pysamerr, "[depad] Have ref '%s' for read '%s'\n", h->target_name[b->core.tid], bam1_qname(b));
+                //fprintf(pysamerr, "[depad] Have ref '%s' for read '%s'\n", h->target_name[b->core.tid], bam_get_qname(b));
             } else if (fai) {
                 if (load_unpadded_ref(fai, h->target_name[b->core.tid], h->target_len[b->core.tid], &r)) {
                     fprintf(pysamerr, "[depad] ERROR: Failed to load '%s' from reference FASTA\n", h->target_name[b->core.tid]);
@@ -243,7 +255,10 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
                 fprintf(pysamerr, "[depad] ERROR: Missing %s embedded reference sequence (and no FASTA file)\n", h->target_name[b->core.tid]);
                 return -1;
             }
-            unpad_seq(b, &q);
+            if (0!=unpad_seq(b, &q)) {
+                fprintf(pysamerr, "[depad] ERROR: Problem parsing SEQ and/or CIGAR in read %s\n", bam_get_qname(b));
+                return -1;
+            };
             if (bam_cigar_op(cigar[0]) == BAM_CSOFT_CLIP) {
                 write_cigar(cigar2, n2, m2, cigar[0]);
             } else if (bam_cigar_op(cigar[0]) == BAM_CHARD_CLIP) {
@@ -259,9 +274,15 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
             if (q.s[0] == BAM_CINS) {
                 for (k = 0; k+1 < b->core.pos && !r.s[b->core.pos - k - 1]; ++k);
                 if (k) write_cigar(cigar2, n2, m2, bam_cigar_gen(k, BAM_CPAD));
+                k = 0;
+            } else if (q.s[0] == BAM_CPAD) {
+                // Join 'k' CPAD to our first cigar op CPAD too.
+                for (k = 0; k+1 < b->core.pos && !r.s[b->core.pos - k - 1]; ++k);
+            } else {
+                k = 0;
             }
             /* Count consecutive CIGAR operators to turn into a CIGAR string */
-            for (i = k = 1, op = q.s[0]; i < q.l; ++i) {
+            for (i = 1, k++, op = q.s[0]; i < q.l; ++i) {
                 if (op != q.s[i]) {
                     write_cigar(cigar2, n2, m2, bam_cigar_gen(k, op));
                     op = q.s[i]; k = 1;
@@ -304,14 +325,14 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
         if (b->core.pos != -1) b->core.pos = posmap[b->core.pos];
         if (b->core.mtid < 0 || b->core.mpos < 0) {
             /* Nice case, no mate to worry about*/
-            // fprintf(pysamerr, "[depad] Read '%s' mate not mapped\n", bam1_qname(b));
+            // fprintf(pysamerr, "[depad] Read '%s' mate not mapped\n", bam_get_qname(b));
             /* TODO - Warning if FLAG says mate should be mapped? */
             /* Clean up funny input where mate position is given but mate reference is missing: */
             b->core.mtid = -1;
             b->core.mpos = -1;
         } else if (b->core.mtid == b->core.tid) {
             /* Nice case, same reference */
-            // fprintf(pysamerr, "[depad] Read '%s' mate mapped to same ref\n", bam1_qname(b));
+            // fprintf(pysamerr, "[depad] Read '%s' mate mapped to same ref\n", bam_get_qname(b));
             b->core.mpos = posmap[b->core.mpos];
         } else {
             /* Nasty case, Must load alternative posmap */
@@ -335,8 +356,10 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
             posmap = update_posmap(posmap, r);
         }
         /* Most reads will have been moved so safest to always recalculate the BIN value */
-        b->core.bin = bam_reg2bin(b->core.pos, bam_calend(&b->core, bam1_cigar(b)));
-        samwrite(out, b);
+        b->core.bin = bam_reg2bin(b->core.pos, bam_endpos(b));
+
+    next_seq:
+        sam_write1(out, h, b);
     }
     if (read_ret < -1) {
         fprintf(pysamerr, "[depad] truncated file.\n");
@@ -347,13 +370,12 @@ int bam_pad2unpad(samfile_t *in, samfile_t *out, faidx_t *fai)
     return ret;
 }
 
-bam_header_t * fix_header(bam_header_t *old, faidx_t *fai)
+bam_hdr_t * fix_header(bam_hdr_t *old, faidx_t *fai)
 {
-#if 0
     int i = 0, unpadded_len = 0;
-    bam_header_t *header = 0 ;
+    bam_hdr_t *header = 0 ;
 
-    header = bam_header_dup(old);
+    header = bam_hdr_dup(old);
     for (i = 0; i < old->n_targets; ++i) {
         unpadded_len = get_unpadded_len(fai, old->target_name[i], old->target_len[i]);
         if (unpadded_len < 0) {
@@ -378,11 +400,54 @@ bam_header_t * fix_header(bam_header_t *old, faidx_t *fai)
         end = strchr(text, '\n');
         assert(end != 0);
         if (text[1]=='S' && text[2]=='Q' && text[3]=='\t') {
-            /* TODO - edit the @SQ line here to remove MD and fix LN. */
-            /* For now just remove the @SQ line, and samtools will */
-            /* automatically generate a minimal replacement with LN. */
-            /* However, that discards any other tags like AS, SP, UR. */
-            //fprintf(pysamerr, "[depad] Removing @SQ line\n");
+            const char *cp = text+3;
+            char *name = strstr(text, "\tSN:");
+            char *name_end;
+            if (!name) {
+                fprintf(pysamerr, "Unable to find SN: header field\n");
+                return NULL;
+            }
+            name += 4;
+            for (name_end = name; name_end != end && *name_end != '\t'; name_end++);
+            strcat(newtext, "@SQ");
+
+            /* Parse the @SQ lines */
+            while (cp != end) {
+                if (end-cp >= 2 && strncmp(cp, "LN", 2) == 0) {
+                    // Rewrite the length
+                    char len_buf[100];
+                    int tid;
+                    for (tid = 0; tid < header->n_targets; tid++) {
+                        // may want to hash this, but new header API incoming.
+                        if (strncmp(name, header->target_name[tid], name_end - name) == 0) {
+                            sprintf(len_buf, "LN:%d", header->target_len[tid]);
+                            strcat(newtext, len_buf);
+                            break;
+                        }
+                    }
+                    while (cp != end && *cp++ != '\t');
+                    if (cp != end)
+                        strcat(newtext, "\t");
+                } else if (end-cp >= 2 &&
+                           (strncmp(cp, "M5", 2) == 0 ||
+                            strncmp(cp, "UR", 2) == 0)) {
+                    // MD5 changed during depadding; ditch it.
+                    // URLs are also invalid.
+                    while (cp != end && *cp++ != '\t');
+                } else {
+                    // Otherwise copy this sub-field verbatim
+                    const char *cp_start = cp;
+                    while (cp != end && *cp++ != '\t');
+                    strncat(newtext, cp_start, cp-cp_start);
+                }
+            }
+
+            // Add newline, replacing trailing '\t' if last on line was the LN:
+            char *text_end = newtext + strlen(newtext);
+            if (text_end[-1] == '\t')
+                text_end[-1] = '\n';
+            else
+                *text_end++ = '\n', *text_end = '\0';
         } else {
             /* Copy this line to the new header */
             strncat(newtext, text, end - text + 1);
@@ -403,41 +468,51 @@ bam_header_t * fix_header(bam_header_t *old, faidx_t *fai)
     }
     //fprintf(pysamerr, "[depad] Here is the new header (pending @SQ lines),\n\n%s\n(end)\n", header->text);
     return header;
-#else
-    fprintf(pysamerr, "Samtools-htslib: fix_header() header parsing not yet implemented\n");
-    abort();
-#endif
 }
 
 static int usage(int is_long_help);
 
 int main_pad2unpad(int argc, char *argv[])
 {
-    samfile_t *in = 0, *out = 0;
-    bam_header_t *h = 0;
+    samFile *in = 0, *out = 0;
+    bam_hdr_t *h = 0, *h_fix = 0;
     faidx_t *fai = 0;
-    int c, is_bamin = 1, compress_level = -1, is_bamout = 1, is_long_help = 0;
-    char in_mode[5], out_mode[5], *fn_out = 0, *fn_list = 0, *fn_ref = 0;
+    int c, compress_level = -1, is_long_help = 0;
+    char in_mode[5], out_mode[6], *fn_out = 0, *fn_list = 0;
     int ret=0;
+    sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
+
+    static const struct option lopts[] = {
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, 0, 0, 'T'),
+        { NULL, 0, NULL, 0 }
+    };
 
     /* parse command-line options */
     strcpy(in_mode, "r"); strcpy(out_mode, "w");
-    while ((c = getopt(argc, argv, "Sso:u1T:?")) >= 0) {
+    while ((c = getopt_long(argc, argv, "SCso:u1T:?", lopts, NULL)) >= 0) {
         switch (c) {
-        case 'S': is_bamin = 0; break;
-        case 's': assert(compress_level == -1); is_bamout = 0; break;
+        case 'S': break;
+        case 'C': hts_parse_format(&ga.out, "cram"); break;
+        case 's': assert(compress_level == -1); hts_parse_format(&ga.out, "sam"); break;
         case 'o': fn_out = strdup(optarg); break;
-        case 'u': assert(is_bamout == 1); compress_level = 0; break;
-        case '1': assert(is_bamout == 1); compress_level = 1; break;
-        case 'T': fn_ref = strdup(optarg); break;
+        case 'u':
+            compress_level = 0;
+            if (ga.out.format == unknown_format)
+                hts_parse_format(&ga.out, "bam");
+            break;
+        case '1':
+            compress_level = 1;
+            if (ga.out.format == unknown_format)
+                hts_parse_format(&ga.out, "bam");
+            break;
         case '?': is_long_help = 1; break;
-        default: return usage(is_long_help);
+        default:  if (parse_sam_global_opt(c, optarg, lopts, &ga) == 0) break;
+            fprintf(pysamerr, "[bam_fillmd] unrecognized option '-%c'\n\n", c);
+            return usage(is_long_help);
         }
     }
     if (argc == optind) return usage(is_long_help);
 
-    if (is_bamin) strcat(in_mode, "b");
-    if (is_bamout) strcat(out_mode, "b");
     strcat(out_mode, "h");
     if (compress_level >= 0) {
         char tmp[2];
@@ -446,47 +521,60 @@ int main_pad2unpad(int argc, char *argv[])
     }
 
     // Load FASTA reference (also needed for SAM -> BAM if missing header)
-    if (fn_ref) {
-        fn_list = samfaipath(fn_ref);
-        fai = fai_load(fn_ref);
+    if (ga.reference) {
+        fn_list = samfaipath(ga.reference);
+        fai = fai_load(ga.reference);
     }
     // open file handlers
-    if ((in = samopen(argv[optind], in_mode, fn_list)) == 0) {
+    if ((in = sam_open_format(argv[optind], in_mode, &ga.in)) == 0) {
         fprintf(pysamerr, "[depad] failed to open \"%s\" for reading.\n", argv[optind]);
         ret = 1;
         goto depad_end;
     }
-    if (in->header == 0) {
+    if (fn_list && hts_set_fai_filename(in, fn_list) != 0) {
+        fprintf(pysamerr, "[depad] failed to load reference file \"%s\".\n", fn_list);
+        ret = 1;
+        goto depad_end;
+    }
+    if ((h = sam_hdr_read(in)) == 0) {
         fprintf(pysamerr, "[depad] failed to read the header from \"%s\".\n", argv[optind]);
         ret = 1;
         goto depad_end;
     }
-    if (in->header->text == 0 || in->header->l_text == 0) {
-        fprintf(pysamerr, "[depad] Warning - failed to read any header text from \"%s\".\n", argv[optind]);
-        assert (0 == in->header->l_text);
-        assert (0 == in->header->text);
-    }
-    if (fn_ref) {
-        h = fix_header(in->header, fai);
+    if (fai) {
+        h_fix = fix_header(h, fai);
     } else {
         fprintf(pysamerr, "[depad] Warning - reference lengths will not be corrected without FASTA reference\n");
-        h = in->header;
+        h_fix = h;
     }
-    if ((out = samopen(fn_out? fn_out : "-", out_mode, h)) == 0) {
+    char wmode[2];
+    strcat(out_mode, sam_open_mode(wmode, fn_out, NULL)==0 ? wmode : "b");
+    if ((out = sam_open_format(fn_out? fn_out : "-", out_mode, &ga.out)) == 0) {
         fprintf(pysamerr, "[depad] failed to open \"%s\" for writing.\n", fn_out? fn_out : "standard output");
         ret = 1;
         goto depad_end;
     }
 
+    // Reference-based CRAM won't work unless we also create a new reference.
+    // We could embed this, but for now we take the easy option.
+    if (ga.out.format == cram)
+        hts_set_opt(out, CRAM_OPT_NO_REF, 1);
+
+    if (sam_hdr_write(out, h_fix) != 0) {
+        fprintf(pysamerr, "[depad] failed to write header.\n");
+        ret = 1;
+        goto depad_end;
+    }
+
     // Do the depad
-    ret = bam_pad2unpad(in, out, fai);
+    ret = bam_pad2unpad(in, out, h, fai);
 
 depad_end:
     // close files, free and return
     if (fai) fai_destroy(fai);
-    if (! (in && h == in->header)) bam_header_destroy(h);
-    samclose(in);
-    samclose(out);
+    if (h) bam_hdr_destroy(h);
+    sam_close(in);
+    sam_close(out);
     free(fn_list); free(fn_out);
     return ret;
 }
@@ -495,14 +583,17 @@ static int usage(int is_long_help)
 {
     fprintf(pysamerr, "\n");
     fprintf(pysamerr, "Usage:   samtools depad <in.bam>\n\n");
-    fprintf(pysamerr, "Options: -s       output is SAM (default is BAM)\n");
-    fprintf(pysamerr, "         -S       input is SAM (default is BAM)\n");
-    fprintf(pysamerr, "         -u       uncompressed BAM output (can't use with -s)\n");
-    fprintf(pysamerr, "         -1       fast compression BAM output (can't use with -s)\n");
-    fprintf(pysamerr, "         -T FILE  padded reference sequence file [null]\n");
-    fprintf(pysamerr, "         -o FILE  output file name [stdout]\n");
-    fprintf(pysamerr, "         -?       longer help\n");
-    fprintf(pysamerr, "\n");
+    fprintf(pysamerr, "Options:\n");
+    fprintf(pysamerr, "  -s           Output is SAM (default is BAM)\n");
+    fprintf(pysamerr, "  -S           Input is SAM (default is BAM)\n");
+    fprintf(pysamerr, "  -u           Uncompressed BAM output (can't use with -s)\n");
+    fprintf(pysamerr, "  -1           Fast compression BAM output (can't use with -s)\n");
+    fprintf(pysamerr, "  -T, --reference FILE\n");
+    fprintf(pysamerr, "               Padded reference sequence file [null]\n");
+    fprintf(pysamerr, "  -o FILE      Output file name [stdout]\n");
+    fprintf(pysamerr, "  -?           Longer help\n");
+    sam_global_opt_help(pysamerr, "-...-");
+
     if (is_long_help)
         fprintf(pysamerr, "Notes:\n\
 \n\
