@@ -1,9 +1,12 @@
 import os
 import pysam
 import unittest
-from TestUtils import checkFieldEqual
+import collections
 import copy
 
+from TestUtils import checkFieldEqual
+
+
 SAMTOOLS = "samtools"
 WORKDIR = "pysam_test_work"
 DATADIR = "pysam_data"
@@ -344,6 +347,26 @@ class TestAlignedSegment(ReadTest):
         self.assertEqual(a.query_alignment_length, 20)
 
 
+class TestAlignedPairs(unittest.TestCase):
+    filename = os.path.join(DATADIR, "example_aligned_pairs.bam")
+
+    def testReferenceBases(self):
+        """reference bases should always be the same nucleotide
+        """
+        reference_bases = collections.defaultdict(list)        
+        with pysam.AlignmentFile(self.filename) as inf:
+            for c in inf.pileup():
+                for r in c.pileups:
+                    for read, ref, base in r.alignment.get_aligned_pairs(
+                            with_seq=True):
+                        if ref is None:
+                            continue
+                        reference_bases[ref].append(base.upper())
+
+        for x, y in reference_bases.items():
+            self.assertEqual(len(set(y)), 1)
+
+
 class TestTags(ReadTest):
 
     def testMissingTag(self):
@@ -463,7 +486,7 @@ class TestTags(ReadTest):
             after = entry.get_tags()
             self.assertEqual(after, before)
 
-    def testMDTag(self):
+    def testMDTagMatchOnly(self):
         a = self.buildRead()
 
         # Substitutions only
@@ -488,23 +511,31 @@ class TestTags(ReadTest):
             "ctgAAAAAcgt",
             a.get_reference_sequence())
 
-        # insertions are silent
+    def testMDTagInsertions(self):
+        a = self.buildRead()
+
+        # insertions are silent in the reference sequence
         a.cigarstring = "5M1I5M"
         a.query_sequence = "A" * 5 + "C" + "A" * 5
-        a.set_tag('MD', "11")
+        a.set_tag('MD', "10")
         self.assertEqual(
-            a.query_sequence,
-            a.get_reference_sequence())
+            a.get_reference_sequence(),
+            "A" * 10)
 
         a.cigarstring = "1I10M"
+        a.query_sequence = "C" * 1 + "A" * 10
         self.assertEqual(
-            a.query_sequence,
-            a.get_reference_sequence())
+            a.get_reference_sequence(),
+            "A" * 10)
 
         a.cigarstring = "10M1I"
+        a.query_sequence = "A" * 10 + "C" * 1
         self.assertEqual(
-            a.query_sequence,
-            a.get_reference_sequence())
+            a.get_reference_sequence(),
+            "A" * 10)
+
+    def testMDTagDeletions(self):
+        a = self.buildRead()
 
         a.cigarstring = "5M1D5M"
         a.query_sequence = "A" * 10
@@ -513,21 +544,24 @@ class TestTags(ReadTest):
             "A" * 5 + "C" + "A" * 5,
             a.get_reference_sequence())
 
-        a.cigarstring = "5M1D5M"
+        a.cigarstring = "5M3D5M"
         a.query_sequence = "A" * 10
         a.set_tag('MD', "5^CCC5")
         self.assertEqual(
             "A" * 5 + "C" * 3 + "A" * 5,
             a.get_reference_sequence())
 
+    def testMDTagSoftClipping(self):
+        a = self.buildRead()
+
         # softclipping
         a.cigarstring = "5S5M1D5M5S"
         a.query_sequence = "G" * 5 + "A" * 10 + "G" * 5
-        a.set_tag('MD', "10")
+        a.set_tag('MD', "5^C5")
         self.assertEqual(
-            "A" * 10,
+            "A" * 5 + "C" + "A" * 5,
             a.get_reference_sequence())
-
+        
         # all together
         a.cigarstring = "5S5M1D5M1I5M5S"
         a.query_sequence = "G" * 5 + "A" * 16 + "G" * 5
@@ -536,13 +570,39 @@ class TestTags(ReadTest):
             "AAcAATAAAAAAAAAA",
             a.get_reference_sequence())
 
-        # all together
-        a.cigarstring = "5S5M1D2I5M5S"
+    def testMDTagComplex(self):
+        a = self.buildRead()
+
+        a.cigarstring = "5S5M1I2D5M5S"
         a.query_sequence = "G" * 5 + "A" * 11 + "G" * 5
         a.set_tag('MD', "2C2^TC5")
         self.assertEqual(
             "AAcAATCAAAAA",
             a.get_reference_sequence())
+        
+        a.cigarstring = "5S5M2D1I5M5S"
+        a.query_sequence = "G" * 5 + "A" * 11 + "G" * 5
+        a.set_tag('MD', "2C2^TC5")
+        self.assertEqual(
+            "AAcAATCAAAAA",
+            a.get_reference_sequence())
+
+        # insertion in reference overlapping deletion in reference
+        # read: AACCCCA---AAA
+        # ref:  AA----AGGGAAA
+        a.cigarstring = "2M4I1M3D3M"
+        a.set_tag("MD", "3^GGG3")
+        a.query_sequence = "AACCCCAAAA"
+        self.assertEqual(
+            "AAAGGGAAA",
+            a.get_reference_sequence())
+
+        a.cigarstring = "5M2D2I2M"
+        a.set_tag("MD", "4C^TT2")
+        a.query_sequence = "A" * 9
+        self.assertEqual(
+            "AAAAcTTAA",
+            a.get_reference_sequence())
 
 
 class TestCopy(ReadTest):
@@ -576,7 +636,7 @@ class TestAsString(unittest.TestCase):
 
     def testAsString(self):
         with open(os.path.join(DATADIR, "ex2.sam")) as samf:
-            reference = [x for x in samf if not x.startswith("@")]
+            reference = [x[:-1] for x in samf if not x.startswith("@")]
 
         with pysam.AlignmentFile(
             os.path.join(DATADIR, "ex2.bam"), "r") as pysamf:
