@@ -5,7 +5,6 @@ Execute in the :file:`tests` directory as it requires the Makefile
 and data files located there.
 '''
 
-import pysam
 import unittest
 import os
 import shutil
@@ -13,14 +12,20 @@ import sys
 import collections
 import subprocess
 import logging
-from functools import partial
-from TestUtils import checkBinaryEqual, checkURL, checkSamtoolsViewEqual, checkFieldEqual
 import array
+if sys.version_info.major >= 3:
+    from io import StringIO
+else:
+    from StringIO import StringIO
+
+from functools import partial
+
+import pysam
+import pysam.samtools
+from TestUtils import checkBinaryEqual, checkURL, \
+    checkSamtoolsViewEqual, checkFieldEqual, force_str
 
-IS_PYTHON3 = sys.version_info[0] >= 3
 
-SAMTOOLS = "samtools"
-WORKDIR = "pysam_test_work"
 DATADIR = "pysam_data"
 
 
@@ -166,12 +171,15 @@ class BasicTestBAMFromFetch(unittest.TestCase):
             self.reads[3].query_sequence, "AGCTTAGCTAGCTACCTATATCTTGGTCTTGGCCG"))
 
     def testARqual(self):
-        self.assertEqual(pysam.qualities_to_qualitystring(self.reads[0].query_qualities), "<<<<<<<<<<<<<<<<<<<<<:<9/,&,22;;<<<",
-                         "quality string mismatch in read 1: %s != %s" % (pysam.qualities_to_qualitystring(self.reads[0].query_qualities), "<<<<<<<<<<<<<<<<<<<<<:<9/,&,22;;<<<"))
-        self.assertEqual(pysam.qualities_to_qualitystring(self.reads[1].query_qualities), "<<<<<;<<<<7;:<<<6;<<<<<<<<<<<<7<<<<", "quality string mismatch in read 2: %s != %s" % (
-            pysam.qualities_to_qualitystring(self.reads[1].query_qualities), "<<<<<;<<<<7;:<<<6;<<<<<<<<<<<<7<<<<"))
-        self.assertEqual(pysam.qualities_to_qualitystring(self.reads[3].query_qualities), "<<<<<<<<<<<<<<<<<<<<<:<9/,&,22;;<<<",
-                         "quality string mismatch in read 3: %s != %s" % (pysam.qualities_to_qualitystring(self.reads[3].query_qualities), "<<<<<<<<<<<<<<<<<<<<<:<9/,&,22;;<<<"))
+        self.assertEqual(
+            pysam.qualities_to_qualitystring(self.reads[0].query_qualities),
+            "<<<<<<<<<<<<<<<<<<<<<:<9/,&,22;;<<<")
+        self.assertEqual(
+            pysam.qualities_to_qualitystring(self.reads[1].query_qualities),
+            "<<<<<;<<<<7;:<<<6;<<<<<<<<<<<<7<<<<")
+        self.assertEqual(
+            pysam.qualities_to_qualitystring(self.reads[3].query_qualities),
+            "<<<<<<<<<<<<<<<<<<<<<:<9/,&,22;;<<<")
 
     def testARquery(self):
         self.assertEqual(
@@ -361,11 +369,27 @@ class BasicTestCRAMFromFile(BasicTestCRAMFromFetch):
 
     def setUp(self):
         f = open(os.path.join(DATADIR, "ex3.cram"))
-        self.samfile = pysam.AlignmentFile(
-            f, "rc")
+        self.samfile = pysam.AlignmentFile(f, "rc")
         self.reads = [r for r in self.samfile]
 
 
+class BasicTestSAMFromStringIO(BasicTestBAMFromFetch):
+
+    def testRaises(self):
+        statement = "samtools view -h {}".format(
+                os.path.join(DATADIR, "ex3.bam"))
+        stdout = subprocess.check_output(statement.split(" "))
+        bam = StringIO()
+        if sys.version_info.major >= 3:
+            bam.write(stdout.decode('ascii'))
+        else:
+            bam.write(stdout)
+        bam.seek(0)
+        self.assertRaises(NotImplementedError,
+                          pysam.AlignmentFile, bam)
+        # self.reads = [r for r in samfile]
+
+
 ##################################################
 #
 # Test of basic File I/O
@@ -405,6 +429,17 @@ class TestIO(unittest.TestCase):
         infile = pysam.AlignmentFile(
             os.path.join(DATADIR, input_filename),
             input_mode)
+
+        if "b" in input_mode:
+            self.assertTrue(infile.is_bam)
+            self.assertFalse(infile.is_cram)
+        elif "c" in input_mode:
+            self.assertFalse(infile.is_bam)
+            self.assertTrue(infile.is_cram)
+        else:
+            self.assertFalse(infile.is_cram)
+            self.assertFalse(infile.is_bam)
+
         if use_template:
             outfile = pysam.AlignmentFile(
                 output_filename,
@@ -686,17 +721,6 @@ class TestIO(unittest.TestCase):
         # write on closed file
         self.assertEqual(0, samfile.write(None))
 
-    def testAutoDetection(self):
-        '''test if autodetection works.'''
-
-        # TODO
-        # samfile = pysam.AlignmentFile(os.path.join(DATADIR, "ex3.sam"))
-        # self.assertRaises(ValueError, samfile.fetch, 'chr1')
-        # samfile.close()
-
-        samfile = pysam.AlignmentFile(os.path.join(DATADIR, "ex3.bam"))
-        samfile.fetch('chr1')
-        samfile.close()
 
     # TOOD
     # def testReadingFromSamFileWithoutHeader(self):
@@ -768,6 +792,62 @@ class TestIO(unittest.TestCase):
         self.assertEqual(samfile.unmapped, 0)
         self.assertEqual(samfile.nocoordinate, 0)
 
+    def testEmptyWithHeaderBAM(self):
+        self.assertRaises(
+            ValueError,
+            pysam.Samfile,
+            os.path.join(DATADIR, "example_empty_with_header.bam"),
+            "rb")
+
+        samfile = pysam.Samfile(
+            os.path.join(DATADIR, "example_empty_with_header.bam"),
+            "rb",
+            check_sq=False)
+        self.assertEqual(samfile.mapped, 0)
+        self.assertEqual(samfile.unmapped, 0)
+        self.assertEqual(samfile.nocoordinate, 0)
+        self.assertEqual([], list(samfile.fetch()))
+
+    def testOpenFromFilename(self):
+
+        samfile = pysam.AlignmentFile(
+            filename=os.path.join(DATADIR, "ex1.bam"),
+            mode="rb")
+        self.assertEqual(len(list(samfile.fetch())), 3270)
+
+
+class TestAutoDetect(unittest.TestCase):
+
+    def testSAM(self):
+        """test SAM autodetection."""
+
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, "ex3.sam")) as inf:
+            self.assertFalse(inf.is_bam)
+            self.assertFalse(inf.is_cram)
+
+            self.assertRaises(ValueError, inf.fetch, 'chr1')
+
+    def testBAM(self):
+        """test BAM autodetection."""
+
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, "ex3.bam")) as inf:
+            self.assertTrue(inf.is_bam)
+            self.assertFalse(inf.is_cram)
+            self.assertEqual(len(list(inf.fetch('chr1'))), 1)
+            self.assertEqual(len(list(inf.fetch('chr2'))), 3)
+
+    def testCRAM(self):
+        """test CRAM autodetection."""
+
+        with pysam.AlignmentFile(
+                os.path.join(DATADIR, "ex3.cram")) as inf:
+            self.assertFalse(inf.is_bam)
+            self.assertTrue(inf.is_cram)
+            self.assertEqual(len(list(inf.fetch('chr1'))), 1)
+            self.assertEqual(len(list(inf.fetch('chr2'))), 3)
+
 
 ##################################################
 #
@@ -787,9 +867,11 @@ class TestIteratorRowBAM(unittest.TestCase):
     def checkRange(self, rnge):
         '''compare results from iterator with those from samtools.'''
         ps = list(self.samfile.fetch(region=rnge))
-        sa = list(pysam.view(self.filename,
-                             rnge,
-                             raw=True))
+        sa = force_str(
+            pysam.samtools.view(
+                self.filename,
+                rnge,
+                raw=True)).splitlines(True)
         self.assertEqual(
             len(ps), len(sa),
             "unequal number of results for range %s: %i != %i" %
@@ -846,15 +928,15 @@ class TestIteratorRowAllBAM(unittest.TestCase):
     def testIterate(self):
         '''compare results from iterator with those from samtools.'''
         ps = list(self.samfile.fetch())
-        sa = list(pysam.view(self.filename,
-                             raw=True))
+        sa = pysam.samtools.view(self.filename,
+                                 raw=True).splitlines()
         self.assertEqual(
             len(ps), len(sa),
             "unequal number of results: %i != %i" %
             (len(ps), len(sa)))
         # check if the same reads are returned
         for line, pair in enumerate(list(zip(ps, sa))):
-            data = pair[1].split("\t")
+            data = force_str(pair[1]).split("\t")
             self.assertEqual(
                 pair[0].query_name,
                 data[0],
@@ -1271,7 +1353,7 @@ class TestHeaderCRAM(TestHeaderSAM):
             self.assertTrue(ak in b, "key '%s' not in '%s' " % (ak, b))
             _strip(b[ak])
 
-            self.assertEqual(sorted(av), sorted(b[ak]))
+            self.assertEqual(av, b[ak])
 
 
 class TestHeaderFromRefs(unittest.TestCase):
@@ -1482,7 +1564,9 @@ class TestWrongFormat(unittest.TestCase):
     def testOpenBamAsSam(self):
         # test fails, needs to be implemented.
         # sam.fetch() fails on reading, not on opening
-        # self.assertRaises( ValueError, pysam.AlignmentFile, 'ex1.bam', 'r' )
+        #self.assertRaises(ValueError, pysam.AlignmentFile,
+        #                  os.path.join(DATADIR, 'ex1.bam'),
+        #                  'r')
         pass
 
     def testOpenFastaAsSam(self):
@@ -1597,8 +1681,10 @@ class TestDeNovoConstruction(unittest.TestCase):
             outfile.write(x)
         outfile.close()
 
-        self.assertTrue(checkBinaryEqual(tmpfilename, self.bamfile),
-                        "mismatch when construction BAM file, see %s %s" % (tmpfilename, self.bamfile))
+        self.assertTrue(
+            checkBinaryEqual(tmpfilename, self.bamfile),
+            "mismatch when construction BAM file, see %s %s" %
+            (tmpfilename, self.bamfile))
 
         os.unlink(tmpfilename)
 
@@ -1807,7 +1893,7 @@ class TestRemoteFileFTP(unittest.TestCase):
         if not checkURL(self.url):
             return
 
-        result = pysam.view(self.url, self.region)
+        result = pysam.samtools.view(self.url, self.region)
         self.assertEqual(len(result), 36)
 
     def testFTPFetch(self):
@@ -1833,8 +1919,8 @@ class TestRemoteFileHTTP(unittest.TestCase):
         samfile_local = pysam.AlignmentFile(self.local, "rb")
         ref = list(samfile_local.fetch(region=self.region))
 
-        result = pysam.view(self.url, self.region)
-        self.assertEqual(len(result), len(ref))
+        result = pysam.samtools.view(self.url, self.region)
+        self.assertEqual(len(result.splitlines()), len(ref))
 
     def testFetch(self):
         if not checkURL(self.url):
@@ -1926,25 +2012,31 @@ class TestPileup(unittest.TestCase):
     def checkEqual(self, references, iterator):
 
         for x, column in enumerate(iterator):
+            v = references[x][:-1].split("\t")
+            self.assertEqual(
+                len(v), 6,
+                "expected 6 values, got {}".format(v))
             (contig, pos, reference_base,
              read_bases, read_qualities, alignment_mapping_qualities) \
-                = references[x][:-1].split("\t")
+                = v
             self.assertEqual(int(pos) - 1, column.reference_pos)
 
     def testSamtoolsStepper(self):
-        refs = pysam.mpileup(
-            "-f", self.fastafilename,
-            self.samfilename)
+        refs = force_str(
+            pysam.samtools.mpileup(
+                "-f", self.fastafilename,
+                self.samfilename)).splitlines(True)
         iterator = self.samfile.pileup(
             stepper="samtools",
             fastafile=self.fastafile)
         self.checkEqual(refs, iterator)
 
     def testAllStepper(self):
-        refs = pysam.mpileup(
-            "-f", self.fastafilename,
-            "-A", "-B",
-            self.samfilename)
+        refs = force_str(
+            pysam.samtools.mpileup(
+                "-f", self.fastafilename,
+                "-A", "-B",
+                self.samfilename)).splitlines(True)
 
         iterator = self.samfile.pileup(
             stepper="all",
@@ -1976,7 +2068,7 @@ class TestCountCoverage(unittest.TestCase):
                 read.flag = read.flag | 0x400
             samfile.write(read)
         samfile.close()
-        pysam.index("test_count_coverage_read_all.bam")
+        pysam.samtools.index("test_count_coverage_read_all.bam")
 
     def count_coverage_python(self, bam, chrom, start, stop,
                               read_callback,
@@ -2109,7 +2201,7 @@ class TestCountCoverage(unittest.TestCase):
                 read.flag = read.flag | 0x400
             samfile.write(read)
         samfile.close()
-        pysam.index("test_count_coverage_nofilter.bam")
+        pysam.samtools.index("test_count_coverage_nofilter.bam")
         samfile = pysam.AlignmentFile("test_count_coverage_nofilter.bam")
         chr = 'chr1'
         start = 0
@@ -2130,6 +2222,26 @@ class TestCountCoverage(unittest.TestCase):
         self.assertEqual(fast_counts[3], manual_counts[3])
 
 
+class TestPileupQueryPosition(unittest.TestCase):
+    
+    filename = "test_query_position.bam"
+
+    def testPileup(self):
+        last = {}
+        with pysam.AlignmentFile(os.path.join(DATADIR, self.filename)) as inf:
+            for col in inf.pileup():
+                for r in col.pileups:
+                    # print r.alignment.query_name
+                    # print r.query_position, r.query_position_or_next, r.is_del
+                    if r.is_del:
+                        self.assertEqual(r.query_position, None)
+                        self.assertEqual(r.query_position_or_next,
+                                         last[r.alignment.query_name] + 1)
+                    else:
+                        self.assertNotEqual(r.query_position, None)
+                        last[r.alignment.query_name] = r.query_position
+
+
 class TestLogging(unittest.TestCase):
 
     '''test around bug issue 42,
@@ -2258,26 +2370,71 @@ class TestAlignmentFileUtilityFunctions(unittest.TestCase):
         self.assertEqual(samfile.nocoordinate, 0)
 
 
+class TestMappedUnmapped(unittest.TestCase):
+    filename = "test_mapped_unmapped.bam"
+
+    def testMapped(self):
+
+        with pysam.AlignmentFile(os.path.join(DATADIR,
+                                              self.filename)) as inf:
+            unmapped_flag = 0
+            unmapped_nopos = 0
+            mapped_flag = 0
+            for x in inf.fetch(until_eof=True):
+                if x.is_unmapped:
+                    if x.reference_id < 0:
+                        unmapped_nopos += 1
+                    else:
+                        unmapped_flag += 1
+                else:
+                    mapped_flag += 1
+
+            self.assertEqual(inf.mapped, mapped_flag)
+            self.assertEqual(inf.unmapped, unmapped_flag + unmapped_nopos)
+
+            inf.reset()
+            self.assertEqual(inf.count(),
+                             inf.mapped + unmapped_flag)
+
+            inf.reset()
+            self.assertEqual(inf.count(until_eof=True),
+                             inf.mapped + unmapped_flag + unmapped_nopos)
+
+            inf.reset()
+            self.assertEqual(inf.count(read_callback="all"),
+                             inf.mapped)
+
+            inf.reset()
+            self.assertEqual(inf.count(until_eof=True, read_callback="all"),
+                             inf.mapped)
+
+
+
 class TestSamtoolsProxy(unittest.TestCase):
 
     '''tests for sanity checking access to samtools functions.'''
 
     def testIndex(self):
-        self.assertRaises(IOError, pysam.index, "missing_file")
+        self.assertRaises(IOError, pysam.samtools.index, "missing_file")
 
     def testView(self):
         # note that view still echos "open: No such file or directory"
-        self.assertRaises(pysam.SamtoolsError, pysam.view, "missing_file")
+        self.assertRaises(pysam.SamtoolsError,
+                          pysam.samtools.view,
+                          "missing_file")
 
     def testSort(self):
-        self.assertRaises(pysam.SamtoolsError, pysam.sort, "missing_file")
+        self.assertRaises(pysam.SamtoolsError,
+                          pysam.samtools.sort,
+                          "missing_file")
 
 
 class TestAlignmentFileIndex(unittest.TestCase):
 
     def testIndex(self):
-        samfile = pysam.AlignmentFile(os.path.join(DATADIR, "ex1.bam"),
-                                      "rb")
+        samfile = pysam.AlignmentFile(
+            os.path.join(DATADIR, "ex1.bam"),
+            "rb")
         index = pysam.IndexedReads(samfile)
         index.build()
         reads = collections.defaultdict(int)
@@ -2292,6 +2449,31 @@ class TestAlignmentFileIndex(unittest.TestCase):
                 self.assertEqual(x.query_name, qname)
 
 
+class TestExplicitIndex(unittest.TestCase):
+
+    def testExplicitIndexBAM(self):
+        samfile = pysam.AlignmentFile(
+            os.path.join(DATADIR, "explicit_index.bam"),
+            "rb",
+            filepath_index=os.path.join(DATADIR, 'ex1.bam.bai'))
+                                      
+        samfile.fetch("chr1")
+
+    def testExplicitIndexCRAM(self):
+        samfile = pysam.AlignmentFile(
+            os.path.join(DATADIR, "explicit_index.cram"),
+            "rc",
+            filepath_index=os.path.join(DATADIR, 'ex1.cram.crai'))
+
+    def testRemoteExplicitIndexBAM(self):
+        samfile = pysam.AlignmentFile(
+            "http://genserv.anat.ox.ac.uk/downloads/pysam/test/noindex.bam",
+            "rb",
+            filepath_index=os.path.join(DATADIR, 'ex1.bam.bai'))
+
+        samfile.fetch("chr1")
+
+
 class TestVerbosity(unittest.TestCase):
 
     '''test if setting/getting of verbosity works.'''
