@@ -35,21 +35,22 @@ def myzip_open(infile, mode="r"):
         return gzip.open(mode)
 
 
-def loadAndConvert(infile, encode=True):
-    '''load data from infile and convert all fields to string.
+def loadAndConvert(filename, encode=True):
+    '''load data from filename and convert all fields to string.
 
-    Infile can be either plain or compressed (ending in .gz).
+    Filename can be either plain or compressed (ending in .gz).
     '''
     data = []
-    if infile.endswith(".gz"):
-        for line in gzip.open(infile):
-            line = line.decode("ascii")
-            if line.startswith("#"):
-                continue
-            d = line.strip().split("\t")
-            data.append(d)
+    if filename.endswith(".gz"):
+        with gzip.open(filename) as inf:
+            for line in inf:
+                line = line.decode("ascii")
+                if line.startswith("#"):
+                    continue
+                d = line.strip().split("\t")
+                data.append(d)
     else:
-        with open(infile) as f:
+        with open(filename) as f:
             for line in f:
                 if line.startswith("#"):
                     continue
@@ -69,10 +70,12 @@ def checkBinaryEqual(filename1, filename2):
     if os.path.getsize(filename1) != os.path.getsize(filename2):
         return False
 
-    infile1 = open(filename1, "rb")
-    infile2 = open(filename2, "rb")
-
-    d1, d2 = infile1.read(), infile2.read()
+    with open(filename1, "rb") as infile:
+        d1 = infile.read()
+       
+    with open(filename2, "rb") as infile:
+        d2 = infile.read()
+ 
     found = False
     for c1, c2 in zip(d1, d2):
         if c1 != c2:
@@ -80,8 +83,6 @@ def checkBinaryEqual(filename1, filename2):
     else:
         found = True
 
-    infile1.close()
-    infile2.close()
     return found
 
 
@@ -112,12 +113,10 @@ class TestCompression(unittest.TestCase):
 
     def setUp(self):
 
-        self.tmpfilename = "tmp_%i" % id(self)
-        infile = gzip.open(self.filename, "rb")
-        outfile = open(self.tmpfilename, "wb")
-        outfile.write(infile.read())
-        outfile.close()
-        infile.close()
+        self.tmpfilename = "tmp_TestCompression_%i" % id(self)
+        with gzip.open(self.filename, "rb") as infile, \
+             open(self.tmpfilename, "wb") as outfile:
+            outfile.write(infile.read())
 
     def testCompression(self):
         '''see also issue 106'''
@@ -176,14 +175,13 @@ class IterationTest(unittest.TestCase):
     def setUp(self):
 
         lines = []
-        inf = gzip.open(self.filename, "rb")
-        for line in inf:
-            line = line.decode('ascii')
-            if line.startswith("#"):
-                if not self.with_comments:
-                    continue
-            lines.append(line)
-        inf.close()
+        with gzip.open(self.filename, "rb") as inf:
+            for line in inf:
+                line = line.decode('ascii')
+                if line.startswith("#"):
+                    if not self.with_comments:
+                        continue
+                lines.append(line)
 
         # creates index of contig, start, end, adds content without newline.
         self.compare = [
@@ -252,11 +250,10 @@ class TestGZFile(IterationTest):
     def setUp(self):
 
         IterationTest.setUp(self)
-
-        self.iter = pysam.GZIterator(self.filename)
+        self.gzfile = pysam.GZIterator(self.filename)
 
     def testAll(self):
-        result = list(self.iter)
+        result = list(self.gzfile)
         ref = self.getSubset()
         self.checkPairwise(result, ref)
 
@@ -350,7 +347,7 @@ class TestIterationWithoutComments(IterationTest):
         # to be implemented
         # self.assertRaises(IndexError, self.tabix.fetch, "chr1", 1000000, 2000000)
 
-        # raise no error for invalid intervals
+        # raise no error for empty intervals
         self.tabix.fetch("chr1", 100, 100)
 
     def testGetContigs(self):
@@ -361,13 +358,12 @@ class TestIterationWithoutComments(IterationTest):
 
     def testHeader(self):
         ref = []
-        inf = gzip.open(self.filename)
-        for x in inf:
-            x = x.decode("ascii")
-            if not x.startswith("#"):
-                break
-            ref.append(x[:-1].encode('ascii'))
-        inf.close()
+        with gzip.open(self.filename) as inf:
+            for x in inf:
+                x = x.decode("ascii")
+                if not x.startswith("#"):
+                    break
+                ref.append(x[:-1].encode('ascii'))
 
         header = list(self.tabix.header)
         self.assertEqual(ref, header)
@@ -376,12 +372,15 @@ class TestIterationWithoutComments(IterationTest):
         '''test repeated opening of the same file.'''
         def func1():
             # opens any tabix file
-            inf = pysam.TabixFile(self.filename)
-            return
-
-        for i in range(10000):
+            with pysam.TabixFile(self.filename) as inf:
+                pass
+            
+        for i in range(1000):
             func1()
 
+    def tearDown(self):
+        self.tabix.close()
+
 
 class TestIterationWithComments(TestIterationWithoutComments):
 
@@ -409,18 +408,21 @@ class TestParser(unittest.TestCase):
     def testRead(self):
 
         for x, r in enumerate(self.tabix.fetch(parser=pysam.asTuple())):
-            self.assertEqual(self.compare[x], list(r))
-            self.assertEqual(len(self.compare[x]), len(r))
+            c = self.compare[x]
+            self.assertEqual(c, list(r))
+            self.assertEqual(len(c), len(r))
 
             # test indexing
-            for c in range(0, len(r)):
-                self.assertEqual(self.compare[x][c], r[c])
+            for y in range(0, len(r)):
+                self.assertEqual(c[y], r[y])
 
             # test slicing access
-            for c in range(0, len(r) - 1):
-                for cc in range(c + 1, len(r)):
-                    self.assertEqual(self.compare[x][c:cc],
-                                     r[c:cc])
+            for y in range(0, len(r) - 1):
+                for cc in range(y + 1, len(r)):
+                    self.assertEqual(c[y:cc],
+                                     r[y:cc])
+            self.assertEqual("\t".join(map(str, c)),
+                             str(r))
 
     def testWrite(self):
 
@@ -471,11 +473,9 @@ class TestParser(unittest.TestCase):
     def testIteratorUncompressed(self):
         '''test iteration from uncompressed file.'''
         tmpfilename = 'tmp_testIteratorUncompressed'
-        infile = gzip.open(self.filename, "rb")
-        outfile = open(tmpfilename, "wb")
-        outfile.write(infile.read())
-        outfile.close()
-        infile.close()
+        with gzip.open(self.filename, "rb") as infile, \
+             open(tmpfilename, "wb") as outfile:
+            outfile.write(infile.read())
 
         with open(tmpfilename) as infile:
             for x, r in enumerate(pysam.tabix_iterator(
@@ -504,7 +504,6 @@ class TestParser(unittest.TestCase):
         b = copy.copy(a)
         self.assertEqual(a, b)
 
-        
 
 class TestIterators(unittest.TestCase):
 
@@ -519,11 +518,9 @@ class TestIterators(unittest.TestCase):
         self.tabix = pysam.TabixFile(self.filename)
         self.compare = loadAndConvert(self.filename)
         self.tmpfilename_uncompressed = 'tmp_TestIterators'
-        infile = gzip.open(self.filename, "rb")
-        outfile = open(self.tmpfilename_uncompressed, "wb")
-        outfile.write(infile.read())
-        outfile.close()
-        infile.close()
+        with gzip.open(self.filename, "rb") as infile, \
+             open(self.tmpfilename_uncompressed, "wb") as outfile:
+            outfile.write(infile.read())
 
     def open(self):
 
@@ -535,21 +532,20 @@ class TestIterators(unittest.TestCase):
 
     def testIteration(self):
 
-        infile = self.open()
+        with self.open() as infile:
+            for x, r in enumerate(self.iterator(infile, self.parser())):
+                self.assertEqual(self.compare[x], list(r))
+                self.assertEqual(len(self.compare[x]), len(r))
 
-        for x, r in enumerate(self.iterator(infile, self.parser())):
-            self.assertEqual(self.compare[x], list(r))
-            self.assertEqual(len(self.compare[x]), len(r))
+                # test indexing
+                for c in range(0, len(r)):
+                    self.assertEqual(self.compare[x][c], r[c])
 
-            # test indexing
-            for c in range(0, len(r)):
-                self.assertEqual(self.compare[x][c], r[c])
-
-            # test slicing access
-            for c in range(0, len(r) - 1):
-                for cc in range(c + 1, len(r)):
-                    self.assertEqual(self.compare[x][c:cc],
-                                     r[c:cc])
+                # test slicing access
+                for c in range(0, len(r) - 1):
+                    for cc in range(c + 1, len(r)):
+                        self.assertEqual(self.compare[x][c:cc],
+                                         r[c:cc])
 
     def testClosedFile(self):
         '''test for error when iterating from closed file.'''
@@ -601,6 +597,8 @@ class TestGTF(TestParser):
             if r.feature != 'gene':
                 self.assertTrue(r.transcript_id.startswith("ENST"))
             self.assertEqual(c[0], r.contig)
+            self.assertEqual("\t".join(map(str, c)),
+                             str(r))
 
 
 class TestIterationMalformattedGTFFiles(unittest.TestCase):
@@ -613,19 +611,23 @@ class TestIterationMalformattedGTFFiles(unittest.TestCase):
 
     def testGTFTooManyFields(self):
 
-        iterator = self.iterator(
-            gzip.open(os.path.join(DATADIR,
-                                   "gtf_toomany_fields.gtf.gz")),
-            parser=self.parser())
-        self.assertRaises(ValueError, iterator.next)
+        with gzip.open(os.path.join(
+                DATADIR,
+                "gtf_toomany_fields.gtf.gz")) as infile:
+            iterator = self.iterator(
+                infile,
+                parser=self.parser())
+            self.assertRaises(ValueError, iterator.next)
 
     def testGTFTooFewFields(self):
 
-        iterator = self.iterator(
-            gzip.open(os.path.join(DATADIR,
-                                   "gtf_toofew_fields.gtf.gz")),
-            parser=self.parser())
-        self.assertRaises(ValueError, iterator.next)
+        with gzip.open(os.path.join(
+                DATADIR,
+                "gtf_toofew_fields.gtf.gz")) as infile:
+            iterator = self.iterator(
+                infile,
+                parser=self.parser())
+            self.assertRaises(ValueError, iterator.next)
 
 
 class TestBed(unittest.TestCase):
@@ -646,6 +648,8 @@ class TestBed(unittest.TestCase):
             self.assertEqual(int(c[1]), r.start)
             self.assertEqual(int(c[2]), r.end)
             self.assertEqual(list(c), list(r))
+            self.assertEqual("\t".join(map(str, c)),
+                             str(r))
 
     def testWrite(self):
 
@@ -666,6 +670,9 @@ class TestBed(unittest.TestCase):
             self.assertEqual(int(c[2]) + 1, r.end)
             self.assertEqual(str(int(c[2]) + 1), r[2])
 
+    def tearDown(self):
+        self.tabix.close()
+
 
 class TestVCF(unittest.TestCase):
 
@@ -697,13 +704,15 @@ if IS_PYTHON3:
         def testFromTabix(self):
 
             # use ascii encoding - should raise error
-            t = pysam.TabixFile(self.tmpfilename + ".gz", encoding="ascii")
-            results = list(t.fetch(parser=pysam.asVCF()))
-            self.assertRaises(UnicodeDecodeError, getattr, results[1], "id")
+            with pysam.TabixFile(
+                    self.tmpfilename + ".gz", encoding="ascii") as t:
+                results = list(t.fetch(parser=pysam.asVCF()))
+                self.assertRaises(UnicodeDecodeError, getattr, results[1], "id")
 
-            t = pysam.TabixFile(self.tmpfilename + ".gz", encoding="utf-8")
-            results = list(t.fetch(parser=pysam.asVCF()))
-            self.assertEqual(getattr(results[1], "id"), u"Rene\xe9")
+            with pysam.TabixFile(
+                    self.tmpfilename + ".gz", encoding="utf-8") as t:
+                results = list(t.fetch(parser=pysam.asVCF()))
+                self.assertEqual(getattr(results[1], "id"), u"Rene\xe9")
 
         def testFromVCF(self):
             self.vcf = pysam.VCF()
@@ -751,6 +760,8 @@ class TestVCFFromTabix(TestVCF):
 
             for y in range(len(c) - ncolumns):
                 self.assertEqual(c[ncolumns + y], r[y])
+            self.assertEqual("\t".join(map(str, c)),
+                             str(r))
 
     def testWrite(self):
 
@@ -793,6 +804,9 @@ class TestVCFFromTabix(TestVCF):
                 c[ncolumns + y] = "test_%i" % y
                 r[y] = "test_%i" % y
                 self.assertEqual(c[ncolumns + y], r[y])
+                
+    def tearDown(self):
+        self.tabix.close()
 
 
 class TestVCFFromVCF(TestVCF):
@@ -813,6 +827,15 @@ class TestVCFFromVCF(TestVCF):
     fail_on_opening = ((24, "Error HEADING_NOT_SEPARATED_BY_TABS"),
                        )
 
+    fail_on_samples = []
+
+    check_samples = False
+    coordinate_offset = 1
+
+    # value returned for missing values
+    missing_value = "."
+    missing_quality = -1
+
     def setUp(self):
 
         TestVCF.setUp(self)
@@ -831,97 +854,180 @@ class TestVCFFromVCF(TestVCF):
             else:
                 self.vcf.connect(self.tmpfilename + ".gz")
 
+    def get_iterator(self):
+
+        f = open(self.filename)
+        fn = os.path.basename(self.filename)
+
+        for x, msg in self.fail_on_opening:
+            if "%i.vcf" % x == fn:
+                self.assertRaises(ValueError, self.vcf.parse, f)
+                return
+
+        return self.vcf.parse(f)
+
+    def get_field_value(self, record, field):
+        return record[field]
+
+    def sample2value(self, r, v):
+        return r, v
+
+    def alt2value(self, r, v):
+        if r == ".":
+            return [], v
+        else:
+            return r.split(","), list(v)
+
+    def filter2value(self, r, v):
+        if r == "PASS":
+            return [], v
+        elif r == ".":
+            return [], v
+        else:
+            return r.split(";"), v
+
     def testParsing(self):
 
+        itr = self.get_iterator()
+        if itr is None:
+            return
+
         fn = os.path.basename(self.filename)
-        with open(self.filename) as f:
 
-            for x, msg in self.fail_on_opening:
-                if "%i.vcf" % x == fn:
-                    self.assertRaises(ValueError, self.vcf.parse, f)
-                    return
-            else:
-                iter = self.vcf.parse(f)
+        for vcf_code, msg in self.fail_on_parsing:
+            if "%i.vcf" % vcf_code == fn:
+                self.assertRaises((ValueError,
+                                   AssertionError),
+                                  list, itr)
+                return
+                # python 2.7
+                # self.assertRaisesRegexp(
+                # ValueError, re.compile(msg), self.vcf.parse, f)
 
-            for x, msg in self.fail_on_parsing:
-                if "%i.vcf" % x == fn:
-                    self.assertRaises(ValueError, list, iter)
-                    break
-                    # python 2.7
-                    # self.assertRaisesRegexp(
-                    # ValueError, re.compile(msg), self.vcf.parse, f)
-            else:
-                # do the actual parsing
-                for x, r in enumerate(iter):
-                    c = self.compare[x]
-                    for y, field in enumerate(self.columns):
-                        # it is ok to have a missing format column
-                        if y == 8 and y == len(c):
-                            continue
+        check_samples = self.check_samples
+        for vcf_code, msg in self.fail_on_samples:
+            if "%i.vcf" % vcf_code == fn:
+                check_samples = False
 
-                        val = r[field]
-                        if field == "pos":
-                            self.assertEqual(int(c[y]) - 1, val)
-                        elif field == "alt":
-                            if c[y] == ".":
-                                # convert . to empty list
-                                self.assertEqual(
-                                    [], val,
-                                    "mismatch in field %s: expected %s, got %s" %
-                                    (field, c[y], val))
-                            else:
-                                # convert to list
-                                self.assertEqual(
-                                    c[y].split(","), val,
-                                    "mismatch in field %s: expected %s, got %s" %
-                                    (field, c[y], val))
+        for x, r in enumerate(itr):
+            c = self.compare[x]
+            for y, field in enumerate(self.columns):
+                # it is ok to have a missing format column
+                if y == 8 and y == len(c):
+                    continue
 
-                        elif field == "filter":
-                            if c[y] == "PASS" or c[y] == ".":
-                                # convert PASS to empty list
-                                self.assertEqual(
-                                    [], val,
-                                    "mismatch in field %s: expected %s, got %s" %
-                                    (field, c[y], val))
-                            else:
-                                # convert to list
-                                self.assertEqual(
-                                    c[y].split(";"), val,
-                                    "mismatch in field %s: expected %s, got %s" %
-                                    (field, c[y], val))
+                val = self.get_field_value(r, field)
+                if field == "pos":
+                    self.assertEqual(int(c[y]) - self.coordinate_offset,
+                                     val)
+                elif field == "alt" or field == "alts":
+                    cc, vv = self.alt2value(c[y], val)
+                    if cc != vv:
+                        # import pdb; pdb.set_trace()
+                        pass
+                    self.assertEqual(
+                        cc, vv,
+                        "mismatch in field %s: expected %s, got %s" %
+                        (field, cc, vv))
 
-                        elif field == "info":
-                            # tests for info field not implemented
-                            pass
-                        elif field == "qual" and c[y] == ".":
-                            self.assertEqual(
-                                -1, val,
-                                "mismatch in field %s: expected %s, got %s" %
-                                (field, c[y], val))
+                elif field == "filter":
+                    cc, vv = self.filter2value(c[y], val)
+                    self.assertEqual(
+                        cc, vv,
+                        "mismatch in field %s: expected %s, got %s" %
+                        (field, cc, vv))
 
-                        elif field == "format":
-                            # format field converted to list
-                            self.assertEqual(
-                                c[y].split(":"), val,
-                                "mismatch in field %s: expected %s, got %s" %
-                                (field, c[y], val))
+                elif field == "info":
+                    # tests for info field not implemented
+                    pass
+
+                elif field == "qual" and c[y] == ".":
+                    self.assertEqual(
+                        self.missing_quality, val,
+                        "mismatch in field %s: expected %s, got %s" %
+                        (field, c[y], val))
+
+                elif field == "format":
+                    # format field converted to list
+                    self.assertEqual(
+                        c[y].split(":"), list(val),
+                        "mismatch in field %s: expected %s, got %s" %
+                        (field, c[y], val))
+
+                elif type(val) in (int, float):
+                    if c[y] == ".":
+                        self.assertEqual(
+                            None, val,
+                            "mismatch in field %s: expected %s, got %s" %
+                            (field, c[y], val))
+                    else:
+                        self.assertAlmostEqual(
+                            float(c[y]), float(val), 2,
+                            "mismatch in field %s: expected %s, got %s" %
+                            (field, c[y], val))
+                else:
+                    if c[y] == ".":
+                        ref_val = self.missing_value
+                    else:
+                        ref_val = c[y]
+                    self.assertEqual(
+                        ref_val, val,
+                        "mismatch in field %s: expected %s(%s), got %s(%s)" %
+                        (field, ref_val, type(ref_val), val, type(val)))
+            # parse samples
+            if check_samples:
+                if len(c) == 8:
+                    for x, s in enumerate(r.samples):
+                        self.assertEqual(
+                            [], r.samples[s].values(),
+                            "mismatch in sample {}: "
+                            "expected [], got {}, src={}, line={}".format(
+                                s, r.samples[s].values(),
+                                r.samples[s].items(), r))
+                else:
+                    for x, s in enumerate(r.samples):
+                        ref, comp = self.sample2value(
+                            c[9 + x],
+                            r.samples[s])
+                        self.compare_samples(ref, comp, s, r)
+
+    def compare_samples(self, ref, comp, s, r):
+
+        if ref != comp:
+
+            # check if GT not at start, not VCF conform and
+            # not supported by cbcf.pyx
+            k = r.format.keys()
+            if "GT" in k and k[0] != "GT":
+                return
+
+            # perform an element-wise checto work around rounding differences
+            for a, b in zip(re.split("[:,;]", ref),
+                            re.split("[:,;]", comp)):
+                is_float = True
+                try:
+                    a = float(a)
+                    b = float(b)
+                except ValueError:
+                    is_float = False
+
+                if is_float:
+                    self.assertAlmostEqual(
+                        a, b, 2,
+                        "mismatch in sample {}: "
+                        "expected {}, got {}, src={}, line={}"
+                        .format(
+                            s, ref, comp,
+                            r.samples[s].items(), r))
+                else:
+                    self.assertEqual(
+                        a, b,
+                        "mismatch in sample {}: "
+                        "expected {}, got {}, src={}, line={}"
+                        .format(
+                            s, ref, comp,
+                            r.samples[s].items(), r))
 
-                        elif type(val) in (int, float):
-                            if c[y] == ".":
-                                self.assertEqual(
-                                    None, val,
-                                    "mismatch in field %s: expected %s, got %s" %
-                                    (field, c[y], val))
-                            else:
-                                self.assertEqual(
-                                    float(c[y]), float(val),
-                                    "mismatch in field %s: expected %s, got %s" %
-                                    (field, c[y], val))
-                        else:
-                            self.assertEqual(
-                                c[y], val,
-                                "mismatch in field %s: expected %s, got %s" %
-                                (field, c[y], val))
 
 ############################################################################
 # create a test class for each example vcf file.
@@ -937,6 +1043,80 @@ for vcf_file in vcf_files:
     globals()[n] = type(n, (TestVCFFromVCF,), dict(filename=vcf_file,))
 
 
+class TestVCFFromVariantFile(TestVCFFromVCF):
+
+    columns = ("chrom", "pos", "id",
+               "ref", "alts", "qual",
+               "filter", "info", "format")
+
+    fail_on_parsing = []
+    fail_on_opening = []
+    coordinate_offset = 0
+    check_samples = True
+    fail_on_samples = [
+        (9, "PL field not defined. Expected to be scalar, but is array"),
+        (12, "PL field not defined. Expected to be scalar, but is array"),
+        (18, "PL field not defined. Expected to be scalar, but is array"),
+    ]
+
+    # value returned for missing values
+    missing_value = None
+    missing_quality = None
+
+    def filter2value(self, r, v):
+        if r == "PASS":
+            return ["PASS"], list(v)
+        elif r == ".":
+            return [], list(v)
+        else:
+            return r.split(";"), list(v)
+
+    def alt2value(self, r, v):
+        if r == ".":
+            return None, v
+        else:
+            return r.split(","), list(v)
+
+    def sample2value(self, r, smp):
+
+        def convert_field(f):
+            if f is None:
+                return "."
+            elif isinstance(f, tuple):
+                return ",".join(map(convert_field, f))
+            else:
+                return str(f)
+
+        v = smp.values()
+
+        if 'GT' in smp:
+            alleles = [str(a) if a is not None else '.' for a in smp.allele_indices]
+            v[0] = '/|'[smp.phased].join(alleles)
+
+        comp = ":".join(map(convert_field, v))
+
+        if comp.endswith(":."):
+            comp = comp[:-2]
+
+        return r, comp
+
+    def setUp(self):
+        TestVCF.setUp(self)
+        self.compare = loadAndConvert(self.filename, encode=False)
+
+    def get_iterator(self):
+        vcf = pysam.VariantFile(self.filename)
+        return vcf.fetch()
+
+    def get_field_value(self, record, field):
+        return getattr(record, field)
+
+
+for vcf_file in vcf_files:
+    n = "TestVCFFromVariantFile_%s" % os.path.basename(vcf_file[:-4])
+    globals()[n] = type(n, (TestVCFFromVariantFile,), dict(filename=vcf_file,))
+
+
 class TestRemoteFileHTTP(unittest.TestCase):
 
     url = "http://genserv.anat.ox.ac.uk/downloads/pysam/test/example_htslib.gtf.gz"
@@ -965,6 +1145,10 @@ class TestRemoteFileHTTP(unittest.TestCase):
                           self.remote_file,
                           "header")
 
+    def tearDown(self):
+        self.remote_file.close()
+        self.local_file.close()
+
 
 class TestIndexArgument(unittest.TestCase):
 
@@ -977,17 +1161,24 @@ class TestIndexArgument(unittest.TestCase):
     def testFetchAll(self):
         shutil.copyfile(self.filename_src, self.filename_dst)
         shutil.copyfile(self.index_src, self.index_dst)
-        same_basename_file = pysam.TabixFile(
-            self.filename_src, "r", index=self.index_src)
-        same_basename_results = list(same_basename_file.fetch())
-        diff_index_file = pysam.TabixFile(
-            self.filename_dst, "r", index=self.index_dst)
-        diff_index_result = list(diff_index_file.fetch())
+
+        with pysam.TabixFile(
+            self.filename_src, "r", index=self.index_src) as \
+            same_basename_file:
+            same_basename_results = list(same_basename_file.fetch())
+
+        with pysam.TabixFile(
+            self.filename_dst, "r", index=self.index_dst) as \
+            diff_index_file:
+            diff_index_result = list(diff_index_file.fetch())
 
         self.assertEqual(len(same_basename_results), len(diff_index_result))
         for x, y in zip(same_basename_results, diff_index_result):
             self.assertEqual(x, y)
 
+        os.unlink(self.filename_dst)
+        os.unlink(self.index_dst)
+
 
 def _TestMultipleIteratorsHelper(filename, multiple_iterators):
     '''open file within scope, return iterator.'''
@@ -999,6 +1190,44 @@ def _TestMultipleIteratorsHelper(filename, multiple_iterators):
     return iterator
 
 
+class TestBackwardsCompatibility(unittest.TestCase):
+    """check if error is raised if a tabix file from an
+    old version is accessed from pysam"""
+
+    def check(self, filename, raises=None):
+        with pysam.TabixFile(filename) as tf:
+            ref = loadAndConvert(filename)
+            if raises is None:
+                self.assertEqual(len(list(tf.fetch())), len(ref))
+            else:
+                self.assertRaises(raises, tf.fetch)
+
+    def testVCF0v23(self):
+        self.check(os.path.join(DATADIR, "example_0v23.vcf.gz"),
+                   ValueError)
+
+    def testBED0v23(self):
+        self.check(os.path.join(DATADIR, "example_0v23.bed.gz"),
+                   ValueError)
+
+    def testVCF0v26(self):
+        self.check(os.path.join(DATADIR, "example_0v26.vcf.gz"),
+                   ValueError)
+
+    def testBED0v26(self):
+        self.check(os.path.join(DATADIR, "example_0v26.bed.gz"),
+                   ValueError)
+
+    def testVCF(self):
+        self.check(os.path.join(DATADIR, "example.vcf.gz"))
+
+    def testBED(self):
+        self.check(os.path.join(DATADIR, "example.bed.gz"))
+
+    def testEmpty(self):
+        self.check(os.path.join(DATADIR, "empty.bed.gz"))
+
+
 class TestMultipleIterators(unittest.TestCase):
 
     filename = os.path.join(DATADIR, "example.gtf.gz")
@@ -1006,22 +1235,22 @@ class TestMultipleIterators(unittest.TestCase):
     def testJoinedIterators(self):
 
         # two iterators working on the same file
-        tabix = pysam.TabixFile(self.filename)
-        a = tabix.fetch(parser=pysam.asGTF()).next()
-        b = tabix.fetch(parser=pysam.asGTF()).next()
-        # the first two lines differ only by the feature field
-        self.assertEqual(a.feature, "UTR")
-        self.assertEqual(b.feature, "exon")
-        self.assertEqual(re.sub("UTR", "", str(a)),
-                         re.sub("exon", "", str(b)))
+        with pysam.TabixFile(self.filename) as tabix:
+            a = tabix.fetch(parser=pysam.asGTF()).next()
+            b = tabix.fetch(parser=pysam.asGTF()).next()
+            # the first two lines differ only by the feature field
+            self.assertEqual(a.feature, "UTR")
+            self.assertEqual(b.feature, "exon")
+            self.assertEqual(re.sub("UTR", "", str(a)),
+                             re.sub("exon", "", str(b)))
 
     def testDisjointIterators(self):
         # two iterators working on the same file
-        tabix = pysam.TabixFile(self.filename)
-        a = tabix.fetch(parser=pysam.asGTF(), multiple_iterators=True).next()
-        b = tabix.fetch(parser=pysam.asGTF(), multiple_iterators=True).next()
-        # both iterators are at top of file
-        self.assertEqual(str(a), str(b))
+        with pysam.TabixFile(self.filename) as tabix:
+            a = tabix.fetch(parser=pysam.asGTF(), multiple_iterators=True).next()
+            b = tabix.fetch(parser=pysam.asGTF(), multiple_iterators=True).next()
+            # both iterators are at top of file
+            self.assertEqual(str(a), str(b))
 
     def testScope(self):
         # technically it does not really test if the scope is correct
@@ -1034,11 +1263,11 @@ class TestMultipleIterators(unittest.TestCase):
 
     def testDoubleFetch(self):
 
-        f = pysam.TabixFile(self.filename)
+        with pysam.TabixFile(self.filename) as f: 
 
-        for a, b in zip(f.fetch(multiple_iterators=True),
-                        f.fetch(multiple_iterators=True)):
-            self.assertEqual(str(a), str(b))
+            for a, b in zip(f.fetch(multiple_iterators=True),
+                            f.fetch(multiple_iterators=True)):
+                self.assertEqual(str(a), str(b))
 
 
 class TestContextManager(unittest.TestCase):
@@ -1052,6 +1281,5 @@ class TestContextManager(unittest.TestCase):
         self.assertEqual(tabixfile.closed, True)
 
 
-
 if __name__ == "__main__":
     unittest.main()
